//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: ExN03PrimaryGeneratorAction.cc,v 1.8 2006/07/11 08:26:06 mackepr Exp $
// GEANT4 tag $Name:  $
//
// 

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "ExN03PrimaryGeneratorAction.hh"

#include "ExN03DetectorConstruction.hh"
#include "ExN03PrimaryGeneratorMessenger.hh"

#include "G4Event.hh"
#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "Randomize.hh"
#include "HistoHelper.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

ExN03PrimaryGeneratorAction::ExN03PrimaryGeneratorAction(
                                             ExN03DetectorConstruction* ExN03DC)
  :ExN03Detector(ExN03DC),rndmFlag("off"),customGunPos(false),flaten(false)
{
  G4int n_particle = 1;
  particleGun  = new G4ParticleGun(n_particle);
  
  //create a messenger for this class
  gunMessenger = new ExN03PrimaryGeneratorMessenger(this);

  // default particle kinematic

  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
  G4String particleName;
  G4ParticleDefinition* particle
    //  = particleTable->FindParticle(particleName="Rmeson+");
    //    = particleTable->FindParticle(particleName="~g_rho+");
    //    = particleTable->FindParticle(particleName="~T+");
    = particleTable->FindParticle(particleName="pi+");
  //                    = particleTable->FindParticle(particleName="e-");
  particleGun->SetParticleDefinition(particle);
  particleGun->SetParticleMomentumDirection(G4ThreeVector(1.,0.,0.));
  //  particleGun->SetParticleEnergy(50.*MeV);
  particleGun->SetParticleEnergy(10.*GeV);
  G4double position = -1.0*(ExN03Detector->GetWorldSizeX());
  G4cout<<"World Size is: "<<ExN03Detector->GetWorldSizeX() / cm<<" cm"<<G4endl;
  particleGun->SetParticlePosition(G4ThreeVector(position,0.*cm,0.*cm));
  theHistoHelper = HistoHelper::GetInstance();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

ExN03PrimaryGeneratorAction::~ExN03PrimaryGeneratorAction()
{
  delete particleGun;
  delete gunMessenger;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void ExN03PrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{
  //this function is called at the begining of event
  // 
  G4double x0 = -1.0*(ExN03Detector->GetWorldSizeX());
  if (customGunPos) x0=xGun;
  G4double y0 = 0.*cm, z0 = 0.*cm;
  if (rndmFlag == "on")
     {y0 = (ExN03Detector->GetWorldSizeYZ())*(G4UniformRand()-0.5);
      z0 = (ExN03Detector->GetWorldSizeYZ())*(G4UniformRand()-0.5);
     } 
  if(flaten) particleGun->SetParticleEnergy(G4UniformRand()*emax);
  theHistoHelper->SaveKey("x0",x0);
  theHistoHelper->SaveKey("Eold",theHistoHelper->GetKey("E0"));
  theHistoHelper->SaveKey("E0",particleGun->GetParticleEnergy());
  if(theHistoHelper->GetKey("EventNumber")==1.E+100) theHistoHelper->SaveKey("EventNumber",0);
  theHistoHelper->SaveKey("EventNumber",theHistoHelper->GetKey("EventNumber")+1);
  particleGun->SetParticlePosition(G4ThreeVector(x0,y0,z0));
  particleGun->GeneratePrimaryVertex(anEvent);
  G4cout<<"Generating energy: "<<particleGun->GetParticleEnergy()/GeV<<" GeV"<<G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void ExN03PrimaryGeneratorAction::SetFlatEn(G4double val)
{
  if (val == 0){
    flaten=false;
  } else {
    flaten=true;
    emax = val;
  }
}
