#include "CustomPhysics.h"
#include "CustomParticleFactory.h"
#include "DummyChargeFlipProcess.h"

#include "G4MultipleScattering.hh"
#include "G4hIonisation.hh"
#include "G4ProcessManager.hh"

#include "G4LeptonConstructor.hh"
#include "G4MesonConstructor.hh"
#include "G4BaryonConstructor.hh"
#include "G4ShortLivedConstructor.hh"
#include "G4IonConstructor.hh"

#include "FullModelHadronicProcess.hh"
#include "ToyModelHadronicProcess.hh"
 
CustomPhysics::CustomPhysics() :  G4VPhysicsConstructor() //   G4VUserPhysicsList() 
{}

CustomPhysics::~CustomPhysics() {}
 
void CustomPhysics::ConstructParticle()
{
    CustomParticleFactory::loadCustomParticles();     
}
 
void CustomPhysics::ConstructProcess() { 
  addCustomPhysics();
}
 
void CustomPhysics::addCustomPhysics()
{
    std::cout << " CustomPhysics: adding CustomPhysics processes  " <<std::endl;
    theParticleIterator->reset();
/*    while((*theParticleIterator)())
    {	
	G4ParticleDefinition* particle = theParticleIterator->value();
	if(CustomParticleFactory::isCustomParticle(particle))
	{
	    std::cout << particle->GetParticleName() << " is Custom" << std::endl;
	    G4ProcessManager* pmanager = particle->GetProcessManager();
	    if(pmanager)
	    { 
              if(particle->GetParticleType()=="rhadron") {
		pmanager->AddDiscreteProcess(new ToyModelHadronicProcess);
	      }
              if(particle->GetPDGCharge()/eplus != 0)
		{ 
//		  pmanager->AddProcess(new G4MultipleScattering,-1, 1,1);
//		  pmanager->AddProcess(new G4hIonisation,       -1, 2,2);

		    std::cout << "    Processes for charged particle added." << std::endl;
                }
                else std::cout << "   It is neutral!!" << std::endl;
               
            }
	    else
		std::cout << "   No pmanager" << std::endl;
	}*/
	

  while((*theParticleIterator)())    {
      int i = 0;
      G4ParticleDefinition* particle = theParticleIterator->value();
      CustomParticle* cp = dynamic_cast<CustomParticle*>(particle);
      if(CustomParticleFactory::isCustomParticle(particle))
        {
          G4cout << particle->GetParticleName()<<", "<<particle->GetPDGEncoding()
                 << " is Custom. Mass is "
                 <<particle->GetPDGMass()/GeV
                 <<" GeV."<<G4endl;
          if(cp->GetCloud()!=0)
            {
              G4cout<<"Cloud mass is "
                    <<cp->GetCloud()->GetPDGMass()/GeV
                    <<" GeV. Spectator mass is "
                    <<static_cast<CustomParticle*>(particle)->GetSpectator()->GetPDGMass()/GeV
                    <<" GeV."       << G4endl;
            }
          G4ProcessManager* pmanager = particle->GetProcessManager();
          if(pmanager)
            {
              if(cp!=0) {
                //              i++;
                //              std::cout<<"1"<<std::endl;
                 pmanager->AddDiscreteProcess(new FullModelHadronicProcess()); //GHEISHA
                //              std::cout<<"2"<<std::endl;
                //              pmanager->AddDiscreteProcess(new ToyModelHadronicProcess()); //Toy model
              }
              if(particle->GetPDGCharge()/eplus != 0)
                {
                  //              std::cout<<"3"<<std::endl;
                 pmanager->AddProcess(new G4MultipleScattering,-1, 1,i+1);
                  //              std::cout<<"4"<<std::endl;
                 pmanager->AddProcess(new G4hIonisation,       -1, 2,i+2);
                  //              std::cout<<"5"<<std::endl;
                  //              G4cout << "    Processes for charged particle added." << G4endl;
                  //              G4cout << " i = "<<i<<G4endl;

                }
              //              else G4cout << "   It is neutral!!" << G4endl;
              pmanager->DumpInfo();
            }
          else      G4cout << "   No pmanager" << G4endl;
        }





	
    }
}
